#!/usr/bin/python3

import serial
import sys
import select
from time import sleep

serialDwell = .1
gpis = {9: 1, 11: 3, 12: 4, 13: 5}
argLen = len(sys.argv)


def setGPI():
    print('Setting GPI(s):')
    for i in range (1, argLen, 2):
        gpi = int(sys.argv[i])
        state = int(sys.argv[i+1])
        print('GPI {} set to value {}'.format(gpi, state))
        if state:
            cmd = 0xb0 | gpi
        else:
            cmd = 0xa0 | gpi
        station.write(cmd.to_bytes(1, 'big'))
        sleep(serialDwell) #give the station time to respond
    print()

def readGPI():
    station.write(b'\x00')
    print('Getting GPI State From Station:')
    sleep(serialDwell)
    if station.in_waiting == 3:
        rx_bytes = []
        for i in range(3):
            rx_bytes.append(station.read())
        #print('Received data: {}'.format(rx_bytes[1]))
    else:
        print('Something went wrong validating GPI state. Try again, state may not be set correctly')
        station.close()
        sys.exit()

    gpiData = int.from_bytes(rx_bytes[1], 'big')

    for key, value in gpis.items():
        if gpiData & (1<<value):
            status = "ON"
        else:
            status = "OFF"
        print('GPI {} = {}'.format(key, status))


if __name__ == '__main__':
    # Configure the serial port
    station = serial.Serial('/dev/ttyS2', 57600, timeout=1, parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, bytesize=serial.EIGHTBITS)
    station.flush()

    # PRINT THE HELP B/C THE WRONG VALUE COUNT FOR WRITING GPI EXISTS IN ARGUMENTS
    if ((argLen % 2) == 0): # also covers if user uses "-h" on the command line
        print('This program requires at least two arguments to take action,\n\
the GPI number (9,11,12,13) and a state (1 or 0). You can change\n\
as many as you like, but it makes little sense to change the same\n\
GPI more than once per issue of this command.\n\n\
Invoking it with no arguments will return the current state of\n\
the station\'s GPIs.\n\n\
Examples:\n\
\t"python gpi_set 9 1" will turn GPI 9 ON\n\
\t"python gpi_set 9 1 12 0" will turn GPI 9 ON and GPI 12 OFF')

    # NO ARGUMENTS, READ AND PRINT GPIS
    elif argLen == 1:
        readGPI()

    # PROPPER ARGUMENT COUNT, SET GPIS THEN VERIFY
    else:
        setGPI()
        readGPI()

    station.close()